#
# Copyright 2019 BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
import argparse
import logging.config
import os
import sys

from cliboa.conf import env
from cliboa.core.factory import ScenarioManagerFactory
from cliboa.core.listener import ScenarioStatusListener
from cliboa.core.worker import ScenarioWorker
from cliboa.util.lisboa_log import LisboaLog


class ScenarioRunner(object):
    """
    Scenario Runner
    """

    def __init__(self, cmd_args):
        """
        Set project directory, scenario file path,
        scenario file format, other command line arguments
        """
        logging.config.fileConfig(
            env.BASE_DIR + "/conf/logging.conf", disable_existing_loggers=False
        )
        self._logger = LisboaLog.get_logger(__name__)
        self._pj_scenario_dir = os.path.join(
            env.PROJECT_DIR, cmd_args.project_name, env.SCENARIO_DIR_NAME
        )
        self._cmd_args = cmd_args

    def add_system_path(self):
        """
        Add system path
        """
        add_target_paths = env.SYSTEM_APPEND_PATHS
        add_target_paths.append(self._pj_scenario_dir)
        for path in add_target_paths:
            sys.path.append(path)

    def create_scenario_queue(self):
        """
        Create scenario queue
        """
        manager = ScenarioManagerFactory.create(self._cmd_args)
        manager.create_scenario_queue()

    def execute_scenario(self):
        """
        Execute scenario
        """
        worker = ScenarioWorker(self._cmd_args)
        worker.register_listeners(ScenarioStatusListener())
        return worker.execute_scenario()


class CommandArgumentParser:
    """
    Parse command line arguments
    """

    def parse(self):
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "project_name", default="hoge", help="Specify execution target project name"
        )
        parser.add_argument("execute_method_argument", nargs="*", default="")
        parser.add_argument(
            "-f",
            "--format",
            nargs="?",
            default="yaml",
            help="Specify yaml or json as FORMAT. Default foramt is yaml",
        )
        args = parser.parse_args()
        return args


def run():
    cmd_parser = CommandArgumentParser()
    cmd_args = cmd_parser.parse()
    runner = ScenarioRunner(cmd_args)
    runner.add_system_path()
    runner.create_scenario_queue()
    return runner.execute_scenario()



import csv


class Rdbms_Util(object):
    def select_sql(self, tblname):
        """
        Create select sql.

        Arguments:
            tblname (str): table name

        Returns:
            str: SELECT * FROM {tblname}
        """
        return "SELECT * FROM %s" % tblname

    def insert_sql(self, tblname, columns):
        """
        Create insert sql from csv file.

        Arguments:
            tblname (str): table name
            columns (array): db column names

        Returns:
            str: INSERT INTO {tblname} (columns) VALUES (%s,%s,%s...)
        """
        item = ""
        for i, row in enumerate(columns):
            if i != 0:
                item = item + ","
            item = item + row
        place_holders = "%s," * (i + 1)
        return "INSERT INTO {} ({}) VALUES ({})".format(tblname, item, place_holders[:-1])

    def delete_sql(self, *args):
        # TODO Not implemented yet
        raise Exception("Not implemented yet")

    def csv_as_params(self, path, chunk_size=100, encoding="utf-8"):
        """
        Divide csv records into small-blocked parameter.
        Basically it uses for paramer of insert query.

        Arguments:
            path (str): Csv file path
            chunk_size=100 (int): Number of row in a single block

        Returns:
            array: [
                    [(row1), (row2), (row3)]
                    [(row4), (row5), (row6)]
                    [(row7), (row8)]
                ]
        """
        tuples = []
        with open(path, mode="r", encoding=encoding) as f:
            reader = csv.reader(f)
            next(reader)  # ignore header
            rows = []
            for i, row in enumerate(reader, 1):
                rows.append(tuple(row))
                if i % chunk_size == 0:
                    tuples.append(rows)
                    rows = []
        if rows:
            tuples.append(rows)
        return tuples



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright BrainPad Inc. All Rights Reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#

import argparse
import os
import sys
from importlib import import_module
from shutil import copyfile

import cliboa


class CliboAdmin(object):
    """
    cliboa administrator command
    """

    def __init__(self, args):
        self._args = args
        self._cmn_dir = None
        self._bin_dir = None

    def main(self):
        """
        Usage:
            cliboadmin init $directory_name
            cliboadmin create $project_name
        """
        if self._args.option == "init":
            self._init_project(self._args.dir_name)
            print("Initialization of cliboa project '" + self._args.dir_name + "' was successful.")
        elif self._args.option == "create":
            self._create_new_project(self._args.dir_name)
            print("Adding a new project '" + self._args.dir_name + "' was successful.")

    def _init_project(self, ini_dir):
        """
        Initialize program configuration
        """
        self._create_ess_dirs(ini_dir)
        self._create_ess_files(ini_dir)

    def _create_ess_dirs(self, ini_dir):
        """
        create essential directories
        """
        os.makedirs(ini_dir, exist_ok=False)
        self._bin_dir = os.path.join(ini_dir, "bin")
        os.makedirs(self._bin_dir, exist_ok=False)
        self._cmn_dir = os.path.join(ini_dir, "common")
        os.makedirs(self._cmn_dir, exist_ok=False)
        os.makedirs(os.path.join(self._cmn_dir, "scenario"), exist_ok=False)
        os.makedirs(os.path.join(ini_dir, "cliboa/conf"), exist_ok=False)
        os.makedirs(os.path.join(ini_dir, "conf"), exist_ok=False)
        os.makedirs(os.path.join(ini_dir, "logs"), exist_ok=False)
        os.makedirs(os.path.join(ini_dir, "project"), exist_ok=False)

    def _create_ess_files(self, ini_dir):
        """
        create essential files
        """
        cliboa_install_path = os.path.dirname(cliboa.__path__[0])

        run_cmd_path = os.path.join(
            cliboa_install_path, "cliboa", "template", "bin", "clibomanager.py"
        )
        copyfile(run_cmd_path, os.path.join(self._bin_dir, "clibomanager.py"))

        # copy Pipfile
        pipfile_path, pyproject_toml_path = self._get_pipfile_and_pyproject_toml_path(
            cliboa_install_path
        )
        copyfile(pipfile_path, os.path.join(ini_dir, "Pipfile"))
        copyfile(pyproject_toml_path, os.path.join(ini_dir, "requirements.txt"))

        # copy environment.py
        cmn_env_path = os.path.join(cliboa_install_path, "cliboa", "conf", "default_environment.py")
        copyfile(cmn_env_path, os.path.join(self._cmn_dir, "environment.py"))

        # copy logging.conf
        conf_path = os.path.join(cliboa_install_path, "cliboa", "conf", "logging.conf")
        copyfile(conf_path, os.path.join(ini_dir, "conf", "logging.conf"))

        # copy cliboa.ini
        conf_path = os.path.join(cliboa_install_path, "cliboa", "conf", "cliboa.ini")
        copyfile(conf_path, os.path.join(ini_dir, "conf", "cliboa.ini"))

        # create __init__.py
        cmn_ini_path = os.path.join(ini_dir, "common", "__init__.py")
        open(cmn_ini_path, "w").close()

    def _create_new_project(self, new_project_dir):
        """
        Create an individual project configuration
        """
        # check if being under cliboa project directory
        sys.path.append(os.getcwd())
        import_module("common.environment")

        # make essential directories and files
        os.makedirs(os.path.join("project", new_project_dir), exist_ok=False)
        os.makedirs(os.path.join("project", new_project_dir, "scenario"), exist_ok=False)
        with open(os.path.join("project", new_project_dir, "scenario.yml"), "w") as yaml:
            yaml.write("scenario:" + "\n")

    def _get_pipfile_and_pyproject_toml_path(self, cliboa_install_path):
        """
        Get path of pyproject.toml and Pipfile for current python version
        """
        py_ver_info = sys.version
        py_ver_info = py_ver_info.split(" ")
        py_ver = py_ver_info[0].split(".")
        py_major_ver = py_ver[0] + "." + py_ver[1]
        py_major_ver_and_pyproject_toml = {
            "3.6": "pyproject.above36.toml",
            "3.7": "pyproject.above37.toml",
            "3.8": "pyproject.above38.toml",
            "3.9": "pyproject.above39.toml",
            "3.10": "pyproject.above310.toml",
        }
        py_major_ver_and_pipfile = {
            "3.5": "Pipfile.above35",
            "3.6": "Pipfile.above36",
            "3.7": "Pipfile.above37",
            "3.8": "Pipfile.above38",
            "3.9": "Pipfile.above39",
            "3.10": "Pipfile.above310",
        }
        pipfile_path = os.path.join(
            cliboa_install_path,
            "cliboa/template",
            py_major_ver_and_pipfile[py_major_ver],
        )
        pyproject_toml_path = os.path.join(
            cliboa_install_path,
            "cliboa/template",
            py_major_ver_and_pyproject_toml[py_major_ver],
        )
        return pipfile_path, pyproject_toml_path


class CommandArgumentParser(object):
    def parse(self):
        """
        Parse cliboadmin arguments
        """
        parser = argparse.ArgumentParser()
        parser.add_argument("option", choices=["init", "create"], help="init or create")
        parser.add_argument(
            "dir_name", help="init $directory_name or create $project_directory_name"
        )
        return parser.parse_args()


def main():
    parser = CommandArgumentParser()
    args = parser.parse()
    admin = CliboAdmin(args)
    admin.main()


if __name__ == "__main__":
    main()



from cliboa.conf import env


class ClassUtil(object):
    def is_custom_cls(self, cls_name):
        """
        Check if given class is a custom class.

        Args:
            cls_name: Class name

        Return:
            True: Custom class
            False: Default class
        """
        custom_classes = env.COMMON_CUSTOM_CLASSES + env.PROJECT_CUSTOM_CLASSES
        for cc in custom_classes:
            split_cc = cc.split(".")
            custom_cls_name = split_cc[1]
            if cls_name == custom_cls_name:
                return True
        return False

    def describe_class(self, cls_name):
        """
        Returns a pair of module path and class name by given name.

        Args:
            cls_name: Class name

        Return (tuple):
            tuple:
                - module path
                - class name
        """
        custom_cls_candidates = env.COMMON_CUSTOM_CLASSES + env.PROJECT_CUSTOM_CLASSES
        module = None
        for c in custom_cls_candidates:
            s = c.split(".")
            if s[-1:][0] == cls_name:
                module = s
                break

        if module is None:
            return None

        root = ".".join(module[:-1])
        mod_name = module[-1:][0]

        return root, mod_name
